package com.bocloud.sms.booter.config;

import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.core.CurrentService;
import com.bocloud.sms.booter.listener.BocloudMessageListener;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {

    private final ConnectionFactory connectionFactory;
    private final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");

    public RabbitMQConfiguration(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     * 声明主题交换
     *
     * @return
     */
    @Bean
    public TopicExchange topicExchange() {
        return new TopicExchange(BoCloudExchange.Default.TOPIC.getExchange(), true, true);
    }

    /**
     * 绑定资源申请流程队列
     *
     * @return
     */
    @Bean
    public Binding bindingMessageQueue(CurrentService service) {
        return BindingBuilder.bind(declareMessageQueue(service)).to(topicExchange()).with("bocloud.message.queue.#");
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    public Queue declareMessageQueue(CurrentService service) {
        return new Queue("bocloud.message.queue.sms." + service.getToken(), true, false, true, arguments);
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        return amqpTemplate;
    }

    /**
     * 声明审计日志队列
     *
     * @return
     */
    @Bean
    public Queue declareLogQueue() {
        return new Queue("bocloud.log.queue", true, false, false, arguments);
    }

    /**
     * 声明通知消息队列
     *
     * @return
     */
    @Bean
    public Queue declareNoticeQueue() {
        return new Queue("event.notice.queue", true, false, false, arguments);
    }

    @Bean
    public RabbitAdmin rabbitAdmin() {
        return new RabbitAdmin(amqpTemplate());
    }

    @Bean
    public SimpleMessageListenerContainer autoMessageListener(CurrentService service, BocloudMessageListener bocloudMessageListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareMessageQueue(service));
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(bocloudMessageListener);
        container.setAmqpAdmin(rabbitAdmin());
        container.setAutoStartup(true);
        return container;
    }
}
