package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Permission;
import com.bocloud.sms.interfaces.PermissionService;
import com.bocloud.sms.model.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 权限控制器
 *
 * @author tyl
 */
@RestController
@RequestMapping("/v1/permissions")
@Api(value = "权限管理", tags = "权限管理")
public class PermissionController {

    private final PermissionService permissionService;

    public PermissionController(PermissionService permissionService) {
        this.permissionService = permissionService;
    }

    /**
     * 查询权限树
     *
     * @param parentId
     * @param tenant
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/tree")
    public BocloudResult<List<PermissionBean>> list(@RequestParam(value = Common.ID, required = false) Long parentId,
                                                @RequestParam(value = "category", required = false) String category,
                                                @RequestParam(value = "tenant", required = false) Boolean tenant) {
        return permissionService.list(parentId, category,tenant);
    }

    /**
     * 查询图标
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询图标", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/icons")
    public BocloudResult<List<PermissionIcon>> listIcon(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.listIcon(requestContext);
    }

    /**
     * 查询所有权限父节点
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询父节点", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/parents")
    public BocloudResult<List<PermissionParentsBean>> listParents(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.listParents(requestContext);
    }

    /**
     * 添加权限
     *
     * @param permissionModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加权限", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody PermissionModel permissionModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.create(permissionModel, requestContext);
    }

    /**
     * 修改权限
     *
     * @param id
     * @param permissionModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "权限修改", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody PermissionModel permissionModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.modify(id, permissionModel, requestContext);
    }

    /**
     * 移除权限
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除权限", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.remove(id, requestContext);
    }

    /**
     * 查看权限详细信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询权限详细信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Permission> detail(@PathVariable(value = Common.ID) Long id) {
        return permissionService.detail(id);
    }

    /**
     * 按类型查询权限列表
     *
     * @param parentId
     * @param tenant
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<List<PermissionBean>> listPermissions(@RequestParam(value = Common.ID, required = false) Long parentId,
                                                               @RequestParam(value = "category", required = false) String category,
                                                               @RequestParam(value = "tenant", required = false) Boolean tenant) {
        return permissionService.list(parentId, category, tenant);
    }


    /**
     * 按字典查询权限列表
     *
     * @param condition
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "字典查询权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/condition")
    public BocloudResult<List<Permission>> listPermissionsByCondition(@RequestParam(value = Common.CONDITION, required = false) String condition) {
        return permissionService.listPermissionsByCondition(condition);
    }

    /**
     * 根据ServiceId获取服务菜单权限信息
     *
     * @param serviceId
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询服务菜单权限", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public BocloudResult<Permission> getServicePermissions(@RequestParam(value = "serviceId", required = false) Long serviceId) {
        return permissionService.getServicePermission(serviceId);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询服务模块API权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/modules")
    public BocloudResult<JSONArray> getApiList() {
        return permissionService.getApiList();
    }

    /**
     * 插件绑定权限
     * @param permissionPlugins
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "插件绑定权限", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/binding/plugins")
    public Result bindingPlugins(@RequestBody PermissionPlugins permissionPlugins,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.bindingPlugins(permissionPlugins.getPermissionIdList(),permissionPlugins.getPluginsCode(),permissionPlugins.getTenant(), requestContext);
    }
}
