package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.ProjectService;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.ProjectModel;
import com.bocloud.sms.model.ProjectUserBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/projects")
@Api(value = "项目关联", tags = "项目管理")
public class ProjectController {

    private final ProjectService projectService;
    private final UserService userService;
    private final StringRedisTemplate redisTemplate;
    private static final Logger logger = LoggerFactory.getLogger(ProjectController.class);

    public ProjectController(ProjectService projectService, UserService userService, StringRedisTemplate redisTemplate) {
        this.projectService = projectService;
        this.userService = userService;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 查询项目列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Project>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.list(pager, requestContext);
    }


    /**
     * 字典查询项目信息
     *
     * @param condition
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "字典查询项目信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/condition")
    public BocloudResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.condition(condition, requestContext);
    }

    /**
     * 创建项目
     *
     * @param projectModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建项目", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody ProjectModel projectModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.create(projectModel, requestContext);
    }

    /**
     * 切换项目
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "切换项目", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}")
    public Result switchProject(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        String apiKey = requestContext.getApiKey();
        Assert.isTrue(StringUtils.isNotEmpty(apiKey), "当前会话存在异常");
        try {
            redisTemplate.opsForValue().set("current_project_" + apiKey, String.valueOf(id));
            return new Result(true, "项目切换成功");
        } catch (Exception e) {
            return new Result(false, "项目切换失败");
        }
    }


    /**
     * 修改项目
     *
     * @param id
     * @param projectModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改项目信息", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody ProjectModel projectModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.modify(id, projectModel, requestContext);
    }

    /**
     * 删除项目
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除项目", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.remove(id, requestContext);
    }


    /**
     * 批量删除项目
     *
     * @param ids
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量删除项目", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping
    public Result batchRemove(@RequestBody List<Long> ids, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.batchRemove(ids, requestContext);
    }

    /**
     * 查看项目详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看项目详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Project> detail(@PathVariable(value = Common.ID) Long id) {
        return projectService.detail(id);
    }

    /**
     * 查看项目所在业务
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看项目所在业务信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/business")
    public BocloudResult<Business> business(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.getBusinessDetail(id, requestContext);
    }

    /**
     * 项目配置用户
     *
     * @param id
     * @param projectUserBean
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "项目配追用户", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/users")
    public Result configUser(@PathVariable(value = Common.ID) Long id, @RequestBody ProjectUserBean projectUserBean) {
        return projectService.configUser(id, projectUserBean);
    }

    /**
     * 项目用户设置为项目经理
     *
     * @param id
     * @param userId
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "项目用户设置为项目经理", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/users/{userId}")
    public Result configUser(@PathVariable(value = Common.ID) Long id, @PathVariable(value = "userId") Long userId) {
        return projectService.configProjectManager(id, userId);
    }

    /**
     * 查询项目下关联的的用户ID
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询项目下关联用户", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/users")
    public BocloudResult<List<User>> listByProject(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listProjectUser(id, requestContext);
    }

    /**
     * 查询项目下关联的的用户ID
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询项目下只关联用户", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/usersIn")
    public BocloudResult<List<User>> listUsersInProject(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listUsersInProject(id, requestContext);
    }


    /**
     * 查询项目经理
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询项目经理", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/managers")
    public BocloudResult<List<User>> listProjectManager(@PathVariable(value = Common.ID) Long id) {
        return userService.listProjectManager(id);
    }

    /**
     * 导出项目列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated
    @ApiOperation(tags = {"CMC", "CSC"}, value = "导出项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                       HttpServletRequest request, HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = projectService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "项目统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            logger.error("export project error,", e);
        }
    }

    @ApiOperation(value = "下载项目列表模板", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletRequest request, HttpServletResponse response,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        projectService.importTemp(response);
    }

    @ApiOperation(value = "导入项目列表", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return projectService.importData(multipartFile, requestContext);
    }
}
