package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.model.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * @author admin
 */
@RestController
@RequestMapping("/v1/tenants")
@Api(value = "租户管理", tags = "租户管理")
public class TenantController {

    private final Logger logger = LoggerFactory.getLogger(TenantController.class);
    private final TenantService tenantService;

    public TenantController(TenantService tenantService) {
        this.tenantService = tenantService;
    }

    /**
     * 添加租户
     *
     * @param tenant
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建租户", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody TenantModel tenant,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.create(tenant, requestContext);
    }

    /**
     * 修改租户
     *
     * @param id
     * @param tenant
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改租户", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody TenantModel tenant,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.modify(id, tenant, requestContext);
    }

    /**
     * 移除租户
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除租户", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.remove(id, requestContext);
    }

    /**
     * 查看租户详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Tenant> detail(@PathVariable(value = Common.ID) Long id) {
        return tenantService.detail(id);
    }

    /**
     * 获取某个租户下的所有项目
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/projects")
    public BocloudResult<List<Project>> projects(@PathVariable(value = Common.ID) Long id) {
        return tenantService.projects(id);
    }


    /**
     * 分页查询租户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "分页获取租户列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Tenant>> list(Pager pager, @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.list(pager, requestContext);
    }

    /**
     * 修改租户状态
     *
     * @param id
     * @param action
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "租户操作", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}")
    public Result updateStatus(@PathVariable(value = Common.ID) Long id,
                               @RequestBody ActionModel action,
                               @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.updateStatus(id, action.getAction(), requestContext);
    }

    /**
     * 修改租户密码
     *
     * @param id             租户id
     * @param passwordModel  密码对象
     * @param requestContext 请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "修改租户密码", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/pwd")
    public Result changePwd(@PathVariable(value = Common.ID) Long id,
                            @RequestBody PasswordModel passwordModel,
                            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.changePwd(id, passwordModel.getOldPassword(), passwordModel.getPassword(), requestContext);
    }

    /**
     * 重置租户密码
     *
     * @param id             租户id
     * @param passwordModel  密码对象
     * @param requestContext 请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "重置租户密码", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/reset")
    public Result resetPwd(@PathVariable(value = Common.ID) Long id,
                           @RequestBody PasswordModel passwordModel,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.resetPwd(id, passwordModel.getPassword(), requestContext);
    }


    /**
     * 检查租户是否异地登录
     *
     * @param id
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "检查租户是否登录", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/status")
    public Result checkTenant(@PathVariable(value = Common.ID) Long id,
                              HttpServletRequest request) {
        return tenantService.checkTenant(id, request.getHeader(Common.TOKEN));
    }

    /**
     * 租户的角色列表
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户的角色列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/roles")
    public BocloudResult<List<Role>> roles(@PathVariable(value = Common.ID) Long id) {
        return tenantService.listRoles(id);
    }

    /**
     * 获取租户安全对象
     *
     * @param apiKey
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取租户安全对象", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/security")
    public BocloudResult<AccountSecurity> security(@RequestParam(value = "apiKey") String apiKey) {
        return tenantService.getByApiKey(apiKey);
    }

    /**
     * 批量操作租户
     *
     * @param batchAction
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量操作租户", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping
    public Result batchLock(@RequestBody BatchActionModel batchAction, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        switch (batchAction.getAction()) {
            case lock:
                return tenantService.batchLock(batchAction.getIds(), requestContext);
            case active:
                return tenantService.batchActive(batchAction.getIds(), requestContext);
            case remove:
                return tenantService.batchRemove(batchAction.getIds(), requestContext);
            default:
                return new Result(false, "不支持这种操作");
        }
    }

    /**
     * 删除租户资源池关系
     *
     * @param poolId
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除租户资源池关联", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/pools")
    public Result poolDelete(@RequestParam(value = "poolId") Long poolId) {
        return tenantService.deleteByPoolId(poolId);
    }


    /**
     * 根据租户查询资源池关联信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户资源池关联", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/pools")
    public BocloudResult<List<TenantPool>> businessPools(@PathVariable(value = Common.ID) Long id) {
        return tenantService.listTenantPool(id);
    }

    /**
     * 租户配置资源池
     *
     * @param id
     * @param poolIds
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户配置资源池", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/pools")
    public Result configPool(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> poolIds, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.configPool(id, null, poolIds, requestContext);
    }

    /**
     * 租户配置业务
     *
     * @param id
     * @param businessIds
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户配置业务", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/businesses")
    public Result configBusiness(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "businessIds") String businessIds) {
        List<Long> businesses = JSONArray.parseArray(businessIds, Long.class);
        return tenantService.configBusiness(id, businesses);
    }

    /**
     * 登录记录查询
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "登录记录查询", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/track")
    public BocloudResult<GridBean<LoginRecord>> records(Pager pager) {
        return this.tenantService.list(pager);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加收藏", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/favorites/{id}")
    public Result addFavorite(@PathVariable("id") Long serviceId, @RequestParam("tenantId") Long tenantId) {
        return tenantService.addFavorite(serviceId, tenantId);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量添加收藏", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/favorites")
    public Result addFavorites(@RequestBody List<Favorite> favorites) {
        return tenantService.addFavorites(favorites);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除收藏", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/favorites")
    public Result cancelTenantFavorite(@RequestParam(value = "serviceId", required = false) Long serviceId, @RequestParam(value = "tenantId", required = false) Long tenantId) {
        return tenantService.cancelFavorite(serviceId, tenantId);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取收藏夹", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/favorites")
    public BocloudResult<List<CloudServiceBean>> listFavorite(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.listFavorite(requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务目录", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/services")
    public BocloudResult<List<ServiceMenuBean>> listServiceMenu(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.listServiceMenu(requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改收藏夹排序", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/favorites/order")
    public Result changePriority(@RequestBody List<PriorityModel> priorityModel,
                                 @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.changePriority(priorityModel, requestContext);
    }

    /**
     * 租户授权服务服务相关角色调整（cos授权服务时调用）
     *
     * @param id
     * @param serviceIds
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户授权角色", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}/roles")
    public Result changeRole(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> serviceIds) {
        return tenantService.modifyServiceRole(id, serviceIds);
    }


    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户下用户", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/users")
    public BocloudResult<List<User>> getTenantUsers(@PathVariable(value = Common.ID) Long id) {
        return tenantService.getTenantUsers(id);
    }

    @ApiOperation(tags = {"CMC"}, value = "租户配置用户", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/users")
    public Result configUsers(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> userIds) {
        return tenantService.configUsers(id, userIds);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户账号是否存在", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{account}/exist")
    public Result exist(@PathVariable(value = Common.ACCOUNT) String account) {
        return tenantService.exist(account);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据省市编码查询租户", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{city}/list")
    public BocloudResult<List<Tenant>> listTenantByCity(@PathVariable(value = "city") String city) {
        return tenantService.listTenantByCity(city);
    }

    /**
     * 导出租户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated
    @ApiOperation(tags = {"CMC", "CSC"}, value = "导出项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                       HttpServletRequest request, HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        Result result = tenantService.export(paramList, requestContext, response);
        Assert.isTrue(result.isSuccess(),"导出租户列表异常");
    }

    @ApiOperation(value = "下载租户列表模板", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletRequest request, HttpServletResponse response,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Result result = tenantService.importTemp("租户列表模板", requestContext, response);
        Assert.isTrue(result.isSuccess(),result.getMessage());
    }

    @ApiOperation(value = "导入租户列表", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return tenantService.importData(multipartFile, requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户账号是否存在", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/condition")
    public BocloudResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition) {
        return tenantService.condition(condition);
    }
}
