package com.bocloud.sms.booter.scheduler;

import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.common.utils.Tokens;
import com.bocloud.service.core.CurrentService;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.SignatureAlgorithm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import static com.bocloud.sms.service.utils.LocalDateTimeUtil.localDateTimeFormat;
import static com.bocloud.sms.service.utils.LocalDateTimeUtil.stringToLocalDateTime;

/**
 * @author tyl
 * @since 2020/11/19 16:30
 */
@Component
public class TokenScheduler implements InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(TokenScheduler.class);
    private final StringRedisTemplate redisTemplate;
    private final CurrentService service;

    public TokenScheduler(StringRedisTemplate redisTemplate, CurrentService service) {
        this.redisTemplate = redisTemplate;
        this.service = service;
    }

    @Scheduled(cron = "*/10 * * * * ?")
    public void checkRefreshToken() {
        if (!service.isLeader()) {
            return;
        }
        Set<String> tokenKeys = this.redisTemplate.keys("token*n");
        if (tokenKeys != null) {
            for (String key : tokenKeys) {
                try {
                    String timeString = (String) redisTemplate.opsForHash().get(key, "refreshTime");
                    if (timeString != null) {
                        LocalDateTime refreshTime = stringToLocalDateTime(timeString);
                        if (refreshTime.isBefore(LocalDateTime.now())) {
                            flush(key);
                        }
                    }
                } catch (Exception e) {
                    logger.error("refresh token error {}, the token key is {}", e.getMessage(), key);
                }
            }
        }
    }

    public void flush(String key) {
        String tokenKey = key.substring(0, key.length() - 2);
        String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
        if (newTokenCache != null && !newTokenCache.isEmpty()) {
            Claims claims = Tokens.parse(newTokenCache);
            String newToken = refreshToken(claims);
            HashMap<String, Object> tokenJson = new HashMap<>(2);
            tokenJson.put("token", newToken);
            tokenJson.put("refreshTime", localDateTimeFormat(LocalDateTime.now().plusMinutes(10)));
            redisTemplate.opsForHash().putAll(tokenKey + "_n", tokenJson);
            this.redisTemplate.expire(tokenKey + "_n", 30L, TimeUnit.MINUTES);
            //原来的token变为旧的token
            this.redisTemplate.opsForValue().set(tokenKey + "_o", newTokenCache);
            this.redisTemplate.expire(tokenKey + "_o", 1L, TimeUnit.MINUTES);
        }
    }

    private static String refreshToken(Claims claims) {
        Map<String, Object> header = MapTools.simpleMap("alg", SignatureAlgorithm.HS256.getValue());
        header.put("typ", "JWT");
        Map<String, Object> payload = MapTools.simpleMap(Common.ACCOUNT, claims.get(Common.ACCOUNT, String.class));
        String catalog = claims.get("catalog", String.class);
        payload.put("catalog", catalog);
        if (RequestContext.Catalog.User.name().equals(catalog)) {
            payload.put(Common.TENANTID, claims.get(Common.TENANTID, Long.class));
            payload.put("tenantName", claims.get("tenantName", String.class));
            payload.put("tenantAccount", claims.get("tenantAccount", String.class));
        }
        payload.put(Common.NAME, claims.get(Common.NAME, String.class));
        payload.put(Common.UUID, claims.get("uuid", Long.class));
        payload.put(Common.API_KEY, claims.get(Common.API_KEY, String.class));

        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MONTH, 3);
        return Tokens.generate(header, payload, calendar.getTime());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.checkRefreshToken();
    }
}
