package com.bocloud.sms.booter.websocket.upload;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.FileSeparator;
import com.bocloud.sms.booter.websocket.model.UploadACK;
import com.bocloud.sms.booter.websocket.model.UploadBean;
import com.bocloud.sms.booter.websocket.model.UploadCancel;
import com.bocloud.sms.booter.websocket.model.UploadMessage;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.*;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class UploadServiceHandler implements WebSocketHandler {

    private static final Logger logger = LoggerFactory.getLogger(UploadServiceHandler.class);
    private static final String SAVE_SUCCESS = "SAVE_SUCCESS";
    private static final String SAVE_FAILURE = "SAVE_FAILURE";
    private static final String CANCEL_SUCCESS = "CANCEL_SUCCESS";
    private static final String CANCEL_FAILURE = "CANCEL_FAILURE";
    private static final String UPLOAD_CANCEL = "UPLOAD_CANCEL";
    private static List<WebSocketSession> currentUsers;
    @Value("${upload.storage.path:/home/cmp}")
    private String path;

    private String getPath() {
        if (this.path.endsWith(File.separatorChar + "")) {
            return this.path.substring(0, this.path.length() - 1);
        }
        return path;
    }

    /**
     * @return the currentUsers
     */
    public static List<WebSocketSession> getCurrentUsers() {
        return currentUsers;
    }

    /**
     * @param currentUsers the currentUsers to set
     */
    public static void setCurrentUsers(List<WebSocketSession> currentUsers) {
        UploadServiceHandler.currentUsers = currentUsers;
    }

    static {
        UploadServiceHandler.setCurrentUsers(new ArrayList<WebSocketSession>());
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        UploadServiceHandler.getCurrentUsers().add(session);
    }

    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
        if (message instanceof TextMessage) {
            handleText(session, (TextMessage) message);
        } else if (message instanceof BinaryMessage) {
            handleBinary(session, (BinaryMessage) message);
        } else {
            logger.error("Unexpected WebSocket message type: " + message);
            throw new IllegalStateException("Unexpected WebSocket message type: " + message);
        }
    }

    /**
     * 处理字符串
     *
     * @param session
     * @param message
     */
    private void handleText(WebSocketSession session, TextMessage message) {
        try {
            if (message.getPayload().contains("upload")) {
                UploadBean uploadBean = JSONObject.parseObject(message.getPayload(), UploadBean.class);
                String timestamp = DateTools.formatTime2String(new Date(), "yyyyMMddHHmmssSSS");
                if (StringUtils.isEmpty(uploadBean.getPath())) {
                    File file = new File(this.getPath() + File.separatorChar + timestamp);
                    if (!file.exists()) {
                        file.mkdirs();
                    }
                    String path = File.separatorChar + timestamp + File.separatorChar + uploadBean.getFilename();
                    uploadBean.setTarget(this.getPath() + path);
                    session.getAttributes().put("uploadBean", uploadBean);
                    session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadACK(path))));
                } else {
                    File file = new File(this.getPath() + uploadBean.getPath() + File.separatorChar + timestamp);
                    if (!file.exists()) {
                        file.mkdirs();
                    }
                    String path = uploadBean.getPath() + File.separatorChar + timestamp + File.separatorChar
                            + uploadBean.getFilename();
                    uploadBean.setTarget(this.getPath() + path);
                    session.getAttributes().put("uploadBean", uploadBean);
                    session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadACK(path))));
                }
            } else if (message.getPayload().contains("sendover")) {
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage("TRUE"))));
            } else if (message.getPayload().contains(UPLOAD_CANCEL)) {
                logger.warn("用户取消文件上传");
                UploadBean uploadBean = (UploadBean) session.getAttributes().get("uploadBean");
                if (deleteFile(uploadBean.getTarget())) {
                    if (deleteDirectory(uploadBean.getTarget())) {
                        session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadCancel(CANCEL_SUCCESS))));
                    }
                } else {
                    session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadCancel(CANCEL_FAILURE))));
                }
                return;
            }
        } catch (JsonParseException e) {
            logger.error("上传文件异常：", e);
        } catch (JsonMappingException e) {
            logger.error("上传文件异常：", e);
        } catch (IOException e) {
            logger.error("上传文件异常：", e);
        }
    }

    @SuppressWarnings("static-access")
    private void handleBinary(WebSocketSession session, BinaryMessage message) throws IOException {
        UploadBean uploadBean = (UploadBean) session.getAttributes().get("uploadBean");
        if (null == uploadBean) {
            return;
        }
        try {
            if (saveFile(message.getPayload(), uploadBean.getTarget())) {
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage(SAVE_SUCCESS))));
            } else {
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage(SAVE_FAILURE))));
                this.deleteFile(uploadBean.getTarget());
            }
        } catch (IOException e) {
            logger.error("上传文件异常：", e);
            session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage(SAVE_FAILURE))));
            this.deleteFile(uploadBean.getTarget());
        }
    }

    /**
     * 将二进制byte[]数组写入文件中
     *
     * @param byteBuffer byte[]数组
     * @param outputFile 文件位置
     * @return 成功: true 失败: false
     */
    private static boolean saveFile(ByteBuffer byteBuffer, String outputFile) {
        FileOutputStream fstream = null;
        File file = null;
        try {
            file = new File(outputFile);
            if (!file.exists() && !file.createNewFile()) {
                logger.error("create new file error,path:{}", file.getPath());
                return false;
            }
            fstream = new FileOutputStream(file, true);
            fstream.write(byteBuffer.array());
        } catch (Exception e) {
            logger.error("saveFile Exception:", e);
            return false;
        } finally {
            if (fstream != null) {
                try {
                    fstream.close();
                } catch (IOException e1) {
                    logger.error("close fileimput Exception:", e1);
                }
            }
        }
        return true;
    }

    private static boolean deleteFile(String filepath) {
        File file = null;
        try {
            file = new File(filepath);
            if (file.exists() && !file.delete()) {
                logger.error("file delete failed, path:{} " + file.getPath());
                return false;
            }
            return true;
        } catch (Exception e) {
            logger.error("Delete file exception:", e);
            return false;
        }
    }

    private static boolean deleteDirectory(String filepath) {
        String[] allPath = filepath.split(FileSeparator.SEPARATOR);
        String dirPath = "";
        for (int i = 0; i < allPath.length - 1; i++) {
            dirPath += File.separator + allPath[i];
        }
        if (!dirPath.endsWith(File.separator)) {
            dirPath += File.separator;
        }
        File dirFile = new File(dirPath);
        if (!dirFile.exists() || !dirFile.isDirectory()) {
            return false;
        }
        return dirFile.delete();

    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        UploadServiceHandler.getCurrentUsers().remove(session);
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        if (session.isOpen()) {
            session.close();
        }
        UploadServiceHandler.getCurrentUsers().remove(session);
    }

    @Override
    public boolean supportsPartialMessages() {
        return false;
    }
}
