package com.bocloud.sms.interfaces;

import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.*;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.model.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.sql.ResultSet;
import java.util.List;
import java.util.Map;

/**
 * 租户抽象Service接口
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.05
 */
public interface TenantService {

    /**
     * 创建租户
     *
     * @param params
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result create(TenantModel params, RequestContext requestContext) throws InternalServerException;

    /**
     * 更新租户
     *
     * @param id
     * @param tenant
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result modify(Long id, TenantModel tenant, RequestContext requestContext) throws InternalServerException;

    /**
     * 删除租户
     *
     * @param id
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result remove(Long id, RequestContext requestContext) throws InternalServerException;

    /**
     * 查看租户详情
     *
     * @param id
     * @return
     */
    BocloudResult<Tenant> detail(Long id);

    /**
     * 分页查询
     *
     * @param pager
     * @param user
     * @return
     */
    BocloudResult<GridBean<Tenant>> list(Pager pager, RequestContext user);

    /**
     * 根据资源池id删除租户资源池关系
     *
     * @param poolId
     * @return
     */
    Result deleteByPoolId(Long poolId);

    /**
     * 租户配置资源池
     *
     * @param id
     * @param category
     * @param poolIds
     * @param requestContext
     * @return
     */
    Result configPool(Long id, String category, List<Long> poolIds, RequestContext requestContext);

    /**
     * 根据租户查询资源池关联信息
     *
     * @param id
     * @return
     */
    BocloudResult<List<TenantPool>> listTenantPool(Long id);

    /**
     * 租户配置资源池
     *
     * @param id
     * @param businessIds
     * @return
     */
    Result configBusiness(Long id, List<Long> businessIds);


    /**
     * 租户登录
     *
     * @param account
     * @param password
     * @param sessionId
     * @param requestIp
     * @throws InternalServerException
     * @return
     */
    BocloudResult<Map<String, Object>> login(String account, String password, String sessionId, String requestIp)
            throws InternalServerException;

    /**
     * 租户退出
     *
     * @param id
     * @throws InternalServerException
     * @return
     */
    Result logout(Long id) throws InternalServerException;

    /**
     * 修改租户状态
     *
     * @param id
     * @param action
     * @param requestContext
     * @return
     */
    Result updateStatus(Long id, ActionModel.Action action, RequestContext requestContext);

    /**
     * 获取租户下的用户
     *
     * @param id
     * @return
     */
    BocloudResult<List<User>> getTenantUsers(Long id);

    /**
     * 修改密码
     *
     * @param id
     * @param oldPassword
     * @param password
     * @param requestContext
     * @return
     */
    Result changePwd(Long id, String oldPassword, String password, RequestContext requestContext);

    /**
     * 重置密码
     *
     * @param id
     * @param password
     * @param requestContext
     * @return
     */
    Result resetPwd(Long id, String password, RequestContext requestContext);

    /**
     * 获取安全密钥(公共方法勿删)
     *
     * @param apiKey
     * @return
     */
    BocloudResult<AccountSecurity> apiKey(String apiKey);

    /**
     * 检查租户是否异地登录
     *
     * @param id
     * @param token
     * @return
     */
    Result checkTenant(Long id, String token);

    /**
     * 查询租户的角色
     *
     * @param tenantId
     * @return
     */
    BocloudResult<List<Role>> listRoles(Long tenantId);

    /**
     * 获取apiKey
     *
     * @param apiKey
     * @return
     */
    BocloudResult<AccountSecurity> getByApiKey(String apiKey);

    /**
     * 根据租户账号冻结租户基本信息
     *
     * @param account
     * @throws InternalServerException
     * @return
     */
    Result lockByAccount(String account) throws InternalServerException;

    /**
     * 批量冻结租户
     *
     * @param ids
     * @param context
     * @return
     */
    Result batchLock(List<Long> ids,RequestContext context);


    /**
     * 批量冻结租户
     *
     * @param ids
     * @param context
     * @return
     */
    Result batchActive(List<Long> ids,RequestContext context);


    /**
     * 批量删除租户
     *
     * @param ids
     * @param context
     * @return
     */
    Result batchRemove(List<Long> ids,RequestContext context);
    /**
     * 保存租户登录数据
     *
     * @param log
     * @throws InternalServerException
     * @return
     */
    Result save(LoginRecord log) throws InternalServerException;

    /**
     * 租户登录记录列表展示
     *
     * @param pager
     * @return
     */
    BocloudResult<GridBean<LoginRecord>> list(Pager pager);

    /**
     * checked租户权限
     *
     * @param context
     * @return
     */
    BocloudResult<Map<String, Object>> listPermissions(RequestContext context);

    /**
     * 校验密码
     *
     * @param id
     * @param password
     * @return
     */
    Result check(Long id, String password);

    /**
     * 批量添加收藏
     *
     * @param favorites
     * @return
     */
    Result addFavorites(List<Favorite> favorites);

    /**
     * 添加收藏
     *
     * @param serviceMenuId
     * @param tenantId
     * @return
     */
    Result addFavorite(Long serviceMenuId, Long tenantId);

    /**
     * 取消收藏
     *
     * @param serviceId
     * @param tenantId
     * @return
     */
    Result cancelFavorite(Long serviceId, Long tenantId);

    /**
     * 获取服务目录
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<ServiceMenuBean>> listServiceMenu(RequestContext requestContext);

    /**
     * 获取收藏夹
     *
     * @param requestContext
     * @return
     */
    BocloudResult<List<CloudServiceBean>> listFavorite(RequestContext requestContext);

    /**
     * 修改收藏夹排序
     *
     * @param params
     * @param requestContext
     * @return
     */
    Result changePriority(List<PriorityModel> params, RequestContext requestContext);

    /**
     * 返回账号相关登录所需信息
     *
     * @param account
     * @return
     */
    BocloudResult<Map<String, Object>> tokenLogin(String account);

    /**
     * 修改默认角色服务关联
     *
     * @param id         租户id
     * @param serviceIds 服务
     * @return
     */
    Result modifyServiceRole(Long id, List<Long> serviceIds);

    /**
     * 获取租户下的项目
     *
     * @param id
     * @return
     */
    BocloudResult<List<Project>> projects(Long id);

    /**
     * 根据账号获取租户信息
     *
     * @param account
     * @return
     */
    Result exist(String account);

    /**
     * 根据省市信息获取租户列表
     *
     * @param city
     * @return
     */
    BocloudResult<List<Tenant>> listTenantByCity(String city);

    /**
     * 租户导入模板
     *
     * @param title
     * @param requestContext
     * @param response
     * @return
     */
    Result importTemp(String title, RequestContext requestContext, HttpServletResponse response);

    /**
     * 租户数据导入
     *
     * @param multipartFile
     * @param requestContext
     * @return
     */
    Result importData(MultipartFile multipartFile, RequestContext requestContext);

    /**
     * 租户配置成员用户
     *
     *
     * @param id
     * @param userIds
     * @return
     */
    Result configUsers(Long id, List<Long> userIds);

    /**
     * 字典查询
     *
     * @param condition
     * @return
     */
    BocloudResult condition(String condition);

    /**
     * 导出租户列表
     *
     * @param paramList
     * @param requestContext
     * @param response
     * @return
     */
    Result export(List<Param> paramList, RequestContext requestContext,HttpServletResponse response);
}
