package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.entity.AccessLog;
import com.bocloud.sms.entity.LogArchiveRecord;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.LogService;
import com.bocloud.sms.model.ServiceInvokeModel;
import com.bocloud.sms.repository.LogArchiveRecordRepository;
import com.bocloud.sms.repository.LogRepository;
import com.bocloud.sms.repository.ServiceInvokeModelRepository;
import com.bocloud.sms.repository.UserRepository;
import org.apache.curator.shaded.com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;

/**
 * 审计日志服务实现类
 *
 * @author dmw
 */
@Service("logService")
public class LogServiceImpl implements LogService {

    private static final Logger logger = LoggerFactory.getLogger(LogServiceImpl.class);
    private static final String REG_EXP = "(?:')|(?:--)|(#+)|(\\\\n)|(/\\*(?:.|[\\n\\r])*?\\*/)";
    private final StringRedisTemplate redisTemplate;
    private final LogRepository logRepository;
    private final LogArchiveRecordRepository logArchiveRecordRepository;
    private final UserRepository userRepository;
    private final ServiceInvokeModelRepository serviceInvokeModelRepository;

    public LogServiceImpl(StringRedisTemplate redisTemplate, LogRepository logRepository, LogArchiveRecordRepository logArchiveRecordRepository, UserRepository userRepository, ServiceInvokeModelRepository serviceInvokeModelRepository) {
        this.redisTemplate = redisTemplate;
        this.logRepository = logRepository;
        this.logArchiveRecordRepository = logArchiveRecordRepository;
        this.userRepository = userRepository;
        this.serviceInvokeModelRepository = serviceInvokeModelRepository;
    }

    @Override
    public BocloudResult<GridBean<AccessLog>> list(Pager pager) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            int total = this.logRepository.count(params);
            List<AccessLog> list = new ArrayList<>();
            if (total > 0) {
                list = this.logRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            }
            for (AccessLog accessLog : list) {
                if (accessLog.getCatalog() != null) {
                    if (accessLog.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                        accessLog.setAccount(redisTemplate.opsForValue().get(accessLog.getCatalog() + "_" + accessLog.getUserId()));
                        accessLog.setUserName(redisTemplate.opsForValue().get(accessLog.getCatalog() + "_Name_" + accessLog.getUserId()));
                    } else {
                        accessLog.setAccount(redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_" + accessLog.getUserId()));
                        accessLog.setUserName(redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_Name_" + accessLog.getUserId()));
                    }
                }
            }
            GridBean<AccessLog> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query accessLog list fail:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public Result create(AccessLog accessLog) {
        try {
            this.logRepository.save(accessLog);
            return new Result(true, "添加成功");
        } catch (Exception e) {
            logger.error("Create accessLog fail:", e);
            return new Result(false, "添加失败", "", "");
        }
    }

    @Override
    public Result batch(List<AccessLog> logs) {
        try {
            logs.forEach(log -> {
                log.setId(UUID.randomUUID().toString());
                if (StringUtils.hasText(log.getDetail())) {
                    log.setDetail(log.getDetail().replaceAll(REG_EXP, ""));
                }
                if (StringUtils.hasText(log.getResult())) {
                    log.setResult(log.getResult().replaceAll(REG_EXP, ""));
                }
            });
            this.logRepository.batch(logs);
            return new Result(true, "success");
        } catch (Exception e) {
            logger.error("batch save log error!", e);
            return new Result(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult<JSONObject> statistics() {
        try {
            JSONObject jobj = new JSONObject();
            List<ServiceInvokeModel> list = serviceInvokeModelRepository.statistics();
            List<Map<String, Object>> nodes = Lists.newArrayList();
            nodes.addAll(createNodes("server", BoCloudService.IMS, BoCloudService.PMS, BoCloudService.SMS,
                    BoCloudService.COP, BoCloudService.COS, BoCloudService.CMS, BoCloudService.COP, BoCloudService.COS));
            nodes.addAll(createNodes("worker", BoCloudService.WORKER));
            jobj.put("nodes", nodes);
            jobj.put("links", list);
            return new BocloudResult<>(true, jobj, "统计服务间调用次数成功");
        } catch (Exception e) {
            logger.error("statistics accessLog fail:", e);
            return new BocloudResult<>(false, "统计服务间调用次数失败");
        }
    }

    private List<Map<String, Object>> createNodes(String type, BoCloudService... services) {
        List<Map<String, Object>> nodes = Lists.newArrayList();
        for (BoCloudService service : services) {
            Map<String, Object> node = MapTools.simpleMap("type", type);
            node.put("name", service.getName());
            nodes.add(node);
        }
        return nodes;
    }

    @Override
    public BocloudResult<StringBuilder> archive(String startTime, String endTime, boolean deleted, RequestContext context, HttpServletRequest request,
                                                HttpServletResponse response) {
        try {
            List<Map<String, Object>> list = logRepository.list(startTime, endTime);
            StringBuilder builder = new StringBuilder();
            Optional.ofNullable(list).orElse(new ArrayList<>())
                    .forEach(log -> builder.append(log.values()).append("\n"));
            if (builder.length() > 0) {
                builder.insert(0, "请求对象  操作  请求IP  响应IP  请求时间  服务名称  请求耗时(ms)  操作者\n");
            }
            LogArchiveRecord record = new LogArchiveRecord();
            User result = userRepository.query(context.getTarget());
            if (result != null) {
                record.setUserName(result.getName());
            }
            record.setUserId(context.getTarget());
            record.setGmtArchive(new Date());
            if (builder.length() > 0) {
                record.setResult("归档成功");
            } else {
                record.setResult("归档失败,此时间段无日志");
            }
            if (deleted) {
                record.setSyncDelete(Boolean.TRUE);
                logRepository.delete(startTime, endTime);
            } else {
                record.setSyncDelete(Boolean.FALSE);
            }
            logArchiveRecordRepository.save(record);
            byte[] bytes = builder.toString().getBytes();
            try {
                //定义输出流
                response.setContentType("text/plain");
                response.setCharacterEncoding("utf-8");
                String filename = "审计日志.txt";
                final String userAgent = request.getHeader("USER-AGENT");
                if (userAgent.contains("Mozilla")) {
                    filename = new String(filename.getBytes(), "ISO8859-1");
                }
                response.setHeader("Content-Disposition", "attachment;fileName=" + filename);
                // 定义输出流
                OutputStream outputStream = response.getOutputStream();
                outputStream.write(bytes);
                outputStream.close();
            } catch (IOException e) {
                logger.error("Log archive error", e);
            }
            return new BocloudResult<>(true, builder, "日志归档成功");
        } catch (Exception e) {
            logger.error("Archiving log has failed!", e);
            return new BocloudResult<>(false, "日志归档失败");
        }
    }
}
