package com.bocloud.sms.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.license.LicenseDaemon;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.entity.Plugins;
import com.bocloud.sms.interfaces.DictService;
import com.bocloud.sms.interfaces.PluginsService;
import com.bocloud.sms.repository.PermissionRepository;
import com.bocloud.sms.repository.PluginsRepository;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 插件表
 *
 * @author fanzx
 * @version 5.6.0-SNAPSHOT
 * @since 2022/2/16
 */
@Service("pluginsService")
public class PluginsServiceImpl implements PluginsService {

    private static final Logger logger = LoggerFactory.getLogger(PluginsServiceImpl.class);
    private final PluginsRepository pluginsRepository;
    private final LockFactory lockFactory;
    private final LicenseDaemon licenseDaemon;
    private final DictService dictService;
    private final PermissionRepository permissionRepository;

    public PluginsServiceImpl(PluginsRepository pluginsRepository, LockFactory lockFactory,
                              CurrentService currentService,DictService dictService,PermissionRepository permissionRepository) {
        this.pluginsRepository = pluginsRepository;
        this.lockFactory = lockFactory;
        this.licenseDaemon = LicenseDaemon.instance(currentService);
        this.dictService = dictService;
        this.permissionRepository = permissionRepository;
    }

    /**
     * 查询plugins列表
     * @param pager
     * @param context
     * @return  插件列表
     */
    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
         GridBean gridBean;
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = this.pluginsRepository.count(pager.getParams(), context);
            List<Plugins> plugins = this.pluginsRepository.list(pager, context);
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, plugins);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("query plugins list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * 创建plugin
     * 创建插件默认都是开启状态
     * @param plugins   插件对象
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult create(Plugins plugins, RequestContext context) {
        try {
            Assert.notNull(plugins, "未获取到任何数据!");
            Plugins dbPlugins = this.pluginsRepository.getByCode(plugins.getCode());
            Plugins dbPluginsByName = this.pluginsRepository.getByName(plugins.getName());
            if (dbPluginsByName!=null){
                return new BocloudResult(false, "插件名称已存在！");
            }
            if (dbPlugins!=null){
                return new BocloudResult(false, "插件编码已存在！");
            }
            plugins.setGmtCreate(new Date());
            plugins.setIsDeleted(false);
            // 默认插件状态就是开启
            plugins.setStatus(true);
            this.pluginsRepository.save(plugins);
            return new BocloudResult(true, "新增成功");
        }catch (Exception e){
            logger.error("create plugin fail:", e);
            return new BocloudResult(false, "新增失败");
        }
    }

    /**
     * 修改plugin
     * 该接口无法修改插件状态
     * @param id    插件唯一id
     * @param plugins 实体类
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult modify(Long id, Plugins plugins, RequestContext context) {
        try {
            Plugins dbPlugins = this.pluginsRepository.get(Plugins.class, id);
            Assert.notNull(dbPlugins, "未查询到任何数据!");
            if (!plugins.getName().equals(dbPlugins.getName())) {
                boolean exists = pluginsRepository.existsSameName(plugins.getName());
                if (exists){
                    return new BocloudResult(false, "插件名称已存在");
                }
            }
            if (!plugins.getCode().equals(dbPlugins.getCode())) {
                boolean exists = pluginsRepository.existsSameCode(plugins.getCode());
                if (exists){
                    return new BocloudResult(false, "插件编码已存在");
                }
            }
            plugins.setGmtModify(new Date());
            plugins.setIsDeleted(false);
            this.pluginsRepository.update(plugins);
            // 修改时需要将权限表绑定的内容同步修改
            permissionRepository.updateByPluginsCode(plugins.getCode(),dbPlugins.getCode());
            return new BocloudResult(true, "插件修改成功");
        }catch (Exception e){
            logger.error("modify plugins fail:", e);
            return new BocloudResult(false, "插件修改失败");
        }
    }

    /**
     * 删除plugin
     * @param id    插件id
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Plugins.class.getSimpleName() + "_" + id;
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Plugins plugins = this.pluginsRepository.get(Plugins.class, id);
            Assert.notNull(plugins, " 数据不存在");
            plugins.setIsDeleted(true);
            this.pluginsRepository.update(plugins);
            // 删除插件时，解绑权限表中所有的菜单
            // 需要将权限表中的插件名称改为""  并将权限表中的插件状态修改为false
            this.permissionRepository.unBindingPlugins(plugins.getCode(),false,null);
            return new BocloudResult(true, "插件删除成功!");

        } catch (Exception e) {
            logger.error("remove plugins error:", e);
            return new BocloudResult(false, "插件删除失败：" + e.getMessage());
        }
    }

    /**
     * 批量删除
     * @param idList    删除插件列表集合
     * @param target
     * @return  成功/失败 个数
     */
    @Override
    public BocloudResult batchRemove(List<Long> idList, Long target) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            Assert.notEmpty(idList, " 至少选择一个插件删除！");
            for (Long id : idList) {
                this.remove(id,new RequestContext());
                rightCount++;
            }
        } catch (Exception e) {
            logger.error("rulegroup batch delete error!", e);
            return new BocloudResult(false, "批量删除插件失败:" + e.getMessage());
        }
        return new BocloudResult(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");
    }

    /**
     * 查询单条plugins
     * @param id    插件id
     * @return      插件详细信息
     */
    @Override
    public BocloudResult detail(Long id) {
        try {
            Plugins plugins = this.pluginsRepository.get(Plugins.class, id);
            if (plugins.getIsDeleted()){
                return new BocloudResult(false, new Plugins(), "查询失败,数据不存在");
            }
            Assert.notNull(plugins, "未查询到任何数据!");
            return new BocloudResult(true, plugins, "查询成功");
        }catch (Exception e){
            logger.error("Query plugins detail error:", e);
            return new BocloudResult(false, new Plugins(), "查询失败" + e.getMessage());
        }
    }

    /**
     * 获取平台列表 to ims
     * @param pager
     * @param context
     * @return  资源纳管时展示的云平台列表
     */
    @Override
    public BocloudResult getVendorList(Pager pager, RequestContext context) {
        try {
            // 获取plugins插件列表
            List<String> plugins = licenseDaemon.content().getPlugins();
            BocloudResult<List<Dictionary>> resourceType = dictService.childTree("RESOURCE_TYPE");
            if (plugins==null){
                return resourceType;
            }
            List<Plugins> pluginsList = this.pluginsRepository.listByCode(plugins);
            List<String> resourceTypeList = new ArrayList<>();
            Optional.ofNullable(pluginsList).orElse(new ArrayList<>()).forEach(plugin->{
                resourceTypeList.add(plugin.getVendorType());
            });
            resourceTypeList.add("SANGFOR");
            List<Dictionary> dictionaries = resourceType.getData();
            if (!ListTool.isEmpty(dictionaries)) {
                for (int i = dictionaries.size() - 1; i >= 0; i--) {
                    Dictionary dictionary = dictionaries.get(i);
                    List<Dictionary> children = dictionary.getChildren();
                    for (int j = children.size() - 1; j >= 0; j--) {
                        Dictionary childrenDictionary = children.get(j);
                        if (!resourceTypeList.contains(childrenDictionary.getValue())) {
                            children.remove(childrenDictionary);
                            continue;
                        }
                    }
                    if (ListTool.isEmpty(children)) {
                        dictionaries.remove(dictionary);
                        continue;
                    }
                }
            }
            // 获取到插件列表中的资源类型
            return new BocloudResult(true, dictionaries, "查询成功");
        }catch (Exception e){
            logger.error("Query plugins vendors error:", e);
            return new BocloudResult(false,"查询失败" + e.getMessage());
        }
    }

    /**
     * 控制插件状态
     * @param id        插件id
     * @param status    插件状态
     * @param context
     * @return  成功/失败
     */
    @Override
    public BocloudResult enabled(Long id, Boolean status, RequestContext context) {
        try {
            // 根据id获取插件详情
            Plugins dbPlugins = this.pluginsRepository.get(Plugins.class, id);
            Assert.notNull(dbPlugins, "未查询到任何数据!");
            // 修改插件状态
            pluginsRepository.updateStatus(id,status);
            // 状态修改完之后 需要同步修改下permission表中插件的状态
            permissionRepository.updateByPluginsStatus(dbPlugins.getCode(),status);
            return new BocloudResult(true, "操作成功");
        }catch (Exception e){
            logger.error("plugin enabled failed:", e);
            return new BocloudResult(false, "操作失败" + e.getMessage());
        }
    }

    /**
     * 获取license中插件列表
     * @param pager
     * @param context
     * @return
     */
    @Override
    public BocloudResult listLicense(Pager pager, RequestContext context) {
        try {
            List<String> plugins = licenseDaemon.content().getPlugins();
            List<Plugins> pluginList = new ArrayList<>();
            Optional.ofNullable(plugins).orElse(new ArrayList<>()).forEach(plugin -> {
                    pluginList.add(new Plugins(plugin,false));
                }
            );
            List<Plugins> dbPluginList = pluginsRepository.list(new Pager(1, Integer.MAX_VALUE, null, null), context);
            logger.info("获取到License中的插件列表为： " + pluginList);
            logger.info("获取到数据库中的插件列表为： " + dbPluginList);
            Optional.ofNullable(dbPluginList).orElse(new ArrayList<>()).forEach(dbPlugin -> {
                pluginList.forEach(plugin -> {
                    if (plugin.getCode().equals(dbPlugin.getCode())){
                        plugin.setId(dbPlugin.getId()!=null ? dbPlugin.getId() : null);
                        plugin.setStatus(dbPlugin.getStatus());
                        plugin.setName(dbPlugin.getName());
                        plugin.setRemark(dbPlugin.getRemark());
                        plugin.setVendorType(dbPlugin.getVendorType());
                    }
                });
            });
            return new BocloudResult(true, pluginList, "查询成功");
        } catch (Exception e) {
            logger.error("query license plugins list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
