package com.bocloud.sms.service.ldap;

import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.repository.DepartmentRepository;
import com.bocloud.sms.repository.SystemConfigRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import java.util.Hashtable;

@Slf4j
@Service
public class LdapAuthService {

    @Autowired
    private SystemConfigRepository systemConfigRepository;

    @Autowired
    private DepartmentRepository departmentRepository;

    @Value("${ldap.loginDn:ou=国金总部,ou=国金证券股份有限公司}")
    private String loginDn;

    public boolean authenticate(User user, String password) {

        SystemConfig urlConfig = systemConfigRepository.queryByCode("LdapUrl");
        String url = urlConfig.getValue();
        Assert.hasText(url, "Ldap地址配置为空");
        SystemConfig baseConfig = systemConfigRepository.queryByCode("LdapBaseDn");
        String baseDn = baseConfig.getValue();
        Assert.hasText(baseDn, "LdapBaseDn配置为空");

        Long departId = user.getDepartId();
        Department department = departmentRepository.queryProxy().col(Department::getId).eq(departId).oneOrNull();
        Assert.notNull(department, "用户部门信息查询失败");
        StringBuilder ou = generateOu(department);
        if (loginDn != null && !loginDn.isEmpty()) {
            ou.append(",").append(loginDn);
        }
        String dn = "uid=" + user.getAccount() + "," + ou + "," + baseDn;
        log.info("ldap登录：{}", dn);
        Hashtable<String, String> env = new Hashtable<>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, url);
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, dn);
        env.put(Context.SECURITY_CREDENTIALS, password);

        try {
            DirContext ctx = new InitialDirContext(env);
            ctx.close();
            return true; // 认证成功
        } catch (AuthenticationException e) {
            log.info("认证失败:",e);
            return false; // 认证失败
        } catch (Exception e) {
            log.info("exception:",e);
            throw new RuntimeException(e);
        }
    }

    private StringBuilder generateOu(Department department) {
        StringBuilder sb = new StringBuilder();
        if (department != null) {
            Long parentId = department.getParentId();
            sb.append("ou=").append(department.getName());
            if (parentId != null && parentId != 0) {
                Department parent = departmentRepository.queryProxy().col(Department::getId).eq(parentId).oneOrNull();
                StringBuilder psb = generateOu(parent);
                if (psb.length() > 0) {
                    sb.append(",").append(psb);
                }
            }
        }
        return sb;
    }


}
