package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 账号安全数据迁移任务
 *
 * @author luyanchao
 * @date 2021-06-18
 */
@Component
public class AccountSecurityTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 依赖UserTask，因为有target_id
     */
    public AccountSecurityTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "账号安全数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 迁移5.0的account_security表中category为Manager和User的数据
        int index = 1;
        String selectSql = "select " + String.join(",", COLUMNS) + " from account_security " +
                "where target_id != 1 and category in ('Manager', 'User')";
        List<Map<String, Object>> muDatas = fromJdbcTemplate.queryForList(selectSql, new HashMap<>());
        for (Map<String, Object> muData : muDatas) {
            String muInsertSql = "insert into account_security (" + String.join(",", COLUMNS) + ") " +
                    "values (" + Arrays.stream(COLUMNS).map(s -> ":" + s).collect(Collectors.joining(",")) + ")";
            targetJdbcTemplate.update(muInsertSql, muData);
            this.percent = Math.floorDiv(index * 40, muDatas.size());
            index++;
        }
        this.percent = 40;

        // 获取5.0到5.5的管理员id对应关系
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        // 获取5.0到5.5的用户id对应关系
        Map<Long, Long> userIdMap = UpgradeUtils.getIdMapOfUser(this.dataSource, this.context.getBean(DataSource.class));
        // 处理account_security表的target_id
        index = 1;
        String selectUserSql = "select " + String.join(",", COLUMNS) + " from account_security  where target_id != 1 and category = 'User' ";
        List<Map<String, Object>> userData = fromJdbcTemplate.queryForList(selectUserSql, new HashMap<>());
        for (Map<String, Object> user : userData) {
            long oldId = Long.parseLong(user.get("id").toString());
            Long newId = userIdMap.get(oldId);
            Map<String, Object> sqlParams = MapTools.simpleMap("oldId", oldId);
            sqlParams.put("newId", newId);
            String sqlUpdate = "update account_security set target_id = :newId where target_id = :oldId ";
            fromJdbcTemplate.update(sqlUpdate, sqlParams);
            this.percent = Math.floorDiv(index * 20, userData.size() + 40);
            index++;
        }
        index = 1;
        String selectManagerSql = "select " + String.join(",", COLUMNS) + " from account_security  where target_id != 1 and category = 'Manager' ";
        List<Map<String, Object>> managerData = fromJdbcTemplate.queryForList(selectManagerSql, new HashMap<>());
        for (Map<String, Object> manager : managerData) {
            long oldId = Long.parseLong(manager.get("id").toString());
            Long newId = idMap.get(oldId);
            Map<String, Object> sqlParams = MapTools.simpleMap("oldId", oldId);
            sqlParams.put("newId", newId);
            String sqlUpdate = "update account_security set target_id = :newId,category = 'User' where target_id = :oldId ";
            fromJdbcTemplate.update(sqlUpdate, sqlParams);
            this.percent = Math.floorDiv(index * 20, managerData.size() + 60);
            index++;
        }
        // 迁移5.0的account_security表中category为Tenant的数据
        index = 1;
        String ttSelectSql = "select " + String.join(",", COLUMNS) + " from account_security " +
                "where category = 'Tenant' ";
        List<Map<String, Object>> ttDatas = fromJdbcTemplate.queryForList(ttSelectSql, new HashMap<>());
        for (Map<String, Object> ttData : ttDatas) {
            String asInsertSql = "insert into account_security (category, " + String.join(",", COLUMNS) + ") " +
                    "values ('Tenant', " + Arrays.stream(COLUMNS).map(s -> ":" + s).collect(Collectors.joining(",")) + ")";
            targetJdbcTemplate.update(asInsertSql, ttData);
            this.percent = Math.floorDiv(index * 20, ttDatas.size()) + 80;
            index++;
        }
        this.percent = 100;
    }

    private static final String[] COLUMNS = new String[]{
            "category", "target_id", "salt", "api_key", "sec_key", "tenant_id"
    };
}
