package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 功能授权数据迁移任务
 *
 * 直接给所有角色所有权限
 * @author luyanchao
 * @date 2021-06-18
 */
@Component
public class ApiPermissionTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param roleTask 依赖RoleTask，因为需要roleId
     */
    @Autowired
    public ApiPermissionTask(ApplicationContext context, RoleTask roleTask) {
        this.context = context;
        this.dependOn.add(roleTask);
    }

    @Override
    public String getName() {
        return "功能授权数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 查询超级管理员角色的api授权
        String selectSqlAdmin = "select " + String.join(",", COLUMNS) + " from api_permission where role_id = 1 ";
        List<Map<String, Object>> adminPermissionDatas = targetJdbcTemplate.queryForList(selectSqlAdmin, new HashMap<>());
        this.percent = 10;

        // 查询租户API角色的api授权，5.5中内置的id=4的角色是租户API角色，但是角色表数据经过数据迁移后租户API角色的id就不一定是4了
        String selectSqlTenant = "select " + String.join(",", COLUMNS) + " from api_permission where role_id = 4 ";
        List<Map<String, Object>> tenantPermissionDatas = targetJdbcTemplate.queryForList(selectSqlTenant, new HashMap<>());
        this.percent = 15;

        // 删掉租户API角色的api授权，因为角色表数据经过数据迁移后租户API角色的id就不一定是4了
        String deleteSqlTenant = "delete from api_permission where role_id = 4";
        targetJdbcTemplate.update(deleteSqlTenant, new HashMap<>());
        this.percent = 20;

        // 查询租户API角色的id，角色表数据经过数据迁移后租户API角色的id就不一定是4了
        String selectSqlTenantId = "select id from role where props = 'tenantAPi' ";
        List<Map<String, Object>> tenantRoleIdData = targetJdbcTemplate.queryForList(selectSqlTenantId, new HashMap<>());
        Long tenantRoleId = Long.parseLong(tenantRoleIdData.get(0).get("id").toString());
        this.percent = 25;

        // 查询所有非5.5内置角色
        String selectRoleSql = "select id from role where id not in (1,2,3,:tenantRoleId) ";
        List<Map<String, Object>> roleDatas = targetJdbcTemplate
                .queryForList(selectRoleSql, Collections.singletonMap("tenantRoleId", tenantRoleId));
        this.percent = 30;

        String insertSql = "insert into api_permission (" + String.join(",", COLUMNS) + ") " +
                "values (" + Arrays.stream(COLUMNS).map(s -> ":" + s).collect(Collectors.joining(",")) + ")";
        // 为所有非5.5内置角色授权
        int index = 1, total = roleDatas.size() * adminPermissionDatas.size();
        for (Map<String, Object> roleData : roleDatas) {
            Long roleId = Long.parseLong(roleData.get("id").toString());
            for (Map<String, Object> permissionData : adminPermissionDatas) {
                Map<String, Object> insertParams = new HashMap<>(permissionData);
                insertParams.put("role_id", roleId);
                targetJdbcTemplate.update(insertSql, insertParams);
                this.percent = Math.floorDiv(index * 40, total) + 30;
                index++;
            }
        }
        this.percent = 70;

        // 为租户API角色授权
        index = 1;
        for (Map<String, Object> permissionData : tenantPermissionDatas) {
            Map<String, Object> insertParams = new HashMap<>(permissionData);
            insertParams.put("role_id", tenantRoleId);
            targetJdbcTemplate.update(insertSql, insertParams);
            this.percent = Math.floorDiv(index * 30, total) + 70;
            index++;
        }
        this.percent = 100;
    }

    private static final String[] COLUMNS = new String[] {
            "role_id","pattern","operation","module"
    };
}
