package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 云主机数据迁移任务
 *
 * @author luyanchao
 * @date 2021-07-01
 */
@Component
public class CloudServerTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public CloudServerTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "云主机数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 先把数据从5.0迁移到5.5
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "cloud_server", COLUMN, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_server", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_server", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cloud_server", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        //  ------------------- 5.5增加了如下字段 -------------------
        //  availability_set_id -- 已经和李振平确认留null即可
        //  availability_zone_ids -- 已经和李振平确认留null即可
        //  backup_count -- 已经和袁明航确认留null即可
        //  bios_uuid -- 已经和袁明航确认留null即可
        //  business_id -- 已经和袁明航确认留null即可
        //  expired_time -- 已经和袁明航确认留null即可
        //  fortress -- 改字段废弃，东吴证券对接堡垒机遗留
        //  license_type -- 已经和李振平确认留null即可
        //  pool_group_id -- 已经和袁明航确认留null即可
        //  resource_group_id -- 已经和袁明航确认留null即可
        //  software -- 已经和魏伟确认留null即可
        //  termination_protected -- 已经和王卫确认留null即可，平台同步会处理该字段
        //  vapp_id -- 王宇
        //  vaults_uuid -- 已经和袁明航确认留null即可
        //  vpc_id -- 已经和袁明航确认留null即可
    }

    private static final String[] COLUMN = new String[] {
            "account","azure_resource_group_name","catalog","category_id","cpu","cpu_hot_enable","cpu_model",
            "creator_id","data_disk","datastores","disk","env","flavor_uuid","gmt_create","gmt_modify","gmt_vmcreate",
            "gmt_vmremove","guid","hmc_cpu","host_id","host_name","id","image_uuid","instance_charge_type",
            "instance_id","instance_network_type","instance_type","internet_bandwidth","internet_charge_type",
            "internet_max_bandwidthIn","internet_max_bandwidthOut","is_deleted","is_recycle","is_template",
            "keypair_name","manager_ip","manager_port","mem_hot_enable","memory","mender_id","model_name","name",
            "network_interface_uuid","network_uuid","os_category","os_name","os_version","owner_id","password",
            "private_ips","project_id","props","public_ips","recycle_time","region","region_id","remark","res_pool_id",
            "resource_group","resource_group_uuid","resource_id","resource_label","router_uuid","security_group_id",
            "sku_id","status","store_id","subnet_uuid","system_disk_category","system_disk_size","template_id",
            "tenant_id","tenant_uuid","tools_config","tools_expired","tools_installed","tools_running","uuid",
            "vdc_id","vendor_id","vendor_type","vpc_uuid","zone"
    };
}
