package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;

/**
 * 图标数据迁移任务
 *
 * 保留5.5的数据，将5.0的数据迁移过来
 * 5.5和5.0的表中，有这些表包含icon字段
 * ci_module     icon_id bigint(20)
 * cloud_service icon_id bigint(20)
 * image         icon_id bigint(20)
 *
 * @author luyanchao
 * @date 2021-07-01
 */
@Component
public class IconTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * 5.5和5.0的表中，有这些表包含icon字段
     * ci_module     icon_id bigint(20)
     * cloud_service icon_id bigint(20)
     * image         icon_id bigint(20)
     * 所以依赖其他Task
     *
     * @param context spring上下文
     * @param ciModuleTask      ci_module     icon_id bigint(20)
     * @param cloudServiceTask  cloud_service icon_id bigint(20)
     * @param imageTask         image         icon_id bigint(20)
     */
    @Autowired
    public IconTask(ApplicationContext context, CiModuleTask ciModuleTask, CloudServiceTask cloudServiceTask, ImageTask imageTask) {
        this.context = context;
        this.dependOn.add(ciModuleTask);
        this.dependOn.add(cloudServiceTask);
        this.dependOn.add(imageTask);
    }

    @Override
    public String getName() {
        return "云主机数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 先把数据从5.0迁移到5.5
        String sqlSelect = "select id, name, category, icon, is_reserved from icon where 1=1";
        List<Map<String, Object>> datas = fromJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
        String sqlInsert = "insert into icon (name, category, icon, is_reserved) " +
                "values (:name, :category, :icon, :is_reserved)";
        int index = 1;
        for (Map<String, Object> data : datas) {
            KeyHolder keyHolder = new GeneratedKeyHolder();
            SqlParameterSource parameterSource = new MapSqlParameterSource(data);
            targetJdbcTemplate.update(sqlInsert, parameterSource, keyHolder);
            long v55Id = Objects.requireNonNull(keyHolder.getKey()).longValue();
            long v50Id = Long.parseLong(data.get("id").toString());
            UpgradeUtils.updateColumn(context.getBean(DataSource.class), " ci_module", "icon_id", v50Id, v55Id);
            UpgradeUtils.updateColumn(context.getBean(DataSource.class), " cloud_service", "icon_id", v50Id, v55Id);
            UpgradeUtils.updateColumn(context.getBean(DataSource.class), " image", "icon_id", v50Id, v55Id);
            this.percent = Math.floorDiv(index * 100, datas.size());
            index++;
        }
        this.percent = 100;
    }
}
