package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 资源配额数据迁移任务
 *
 * @author luyanchao
 * @date 2021-06-18
 */
@Component
public class ResourceQuotaTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 依赖UserTask，因为有target_id
     */
    public ResourceQuotaTask(ApplicationContext context, UserTask userTask, CloudServiceTask serviceTask) {
        this.context = context;
        this.dependOn.add(userTask);
        this.dependOn.add(serviceTask);
    }

    @Override
    public String getName() {
        return "资源配额数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 迁移5.0的resource_quota表中catalog为Tenant类型的数据
        int index = 1;
        String selectSql = "select " + String.join(",", COLUMNS) + " from resource_quota " +
                "where catalog = 'TENANT'";
        String serviceSql = "select code from cloud_service";
        List<Map<String, Object>> muDatas = fromJdbcTemplate.queryForList(selectSql, new HashMap<>());
        List<Map<String, Object>> serviceDatas = fromJdbcTemplate.queryForList(serviceSql, new HashMap<>());
        List<String> serviceCodes = serviceDatas.stream().flatMap(stringObjectMap -> stringObjectMap.entrySet().stream())
                .map(map -> map.getValue().toString()).collect(Collectors.toList());
        for (Map<String, Object> muData : muDatas) {
            long assigned = 0L;
            if (muData.get("assigned") != null) {
                assigned = Long.parseLong(muData.get("assigned").toString());
            }
            long used = 0L;
            if (muData.get("used") != null) {
                used = Long.parseLong(muData.get("used").toString());
            }
            muData.put("assigned", 0);
            muData.put("used", used + assigned);
        }
        for (Map<String, Object> muData : muDatas) {
            if (muData.get("code") != null) {
                String code = muData.get("code").toString();
                //没有的服务配额直接跳过
                if (!code.startsWith("meta") && !serviceCodes.contains(code)) {
                    continue;
                }
            }
            String muInsertSql = "insert into resource_quota (" + String.join(",", COLUMNS) + ") " +
                    "values (" + Arrays.stream(COLUMNS).map(s -> ":" + s).collect(Collectors.joining(",")) + ")";
            targetJdbcTemplate.update(muInsertSql, muData);
            this.percent = Math.floorDiv(index * 100, muDatas.size());
            index++;
        }
        this.percent = 100;
    }

    private static final String[] COLUMNS = new String[]{
            "target", "catalog", "quota", "assigned", "used", "code", "unit", "category"
    };
}
