package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;

/**
 * 租户数据迁移任务
 *
 * @author luyanchao
 * @date 2021-06-18
 */
@Component
public class TenantTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public TenantTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "租户数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate jdbcTemplate = this.context.getBean(NamedParameterJdbcTemplate.class);
        // 迁移租户数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "tenant", TENANT_COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        // 设置有所租户启用配额
        String updateSql = "update tenant set quota_able = 1 where 1=1";
        jdbcTemplate.update(updateSql, new HashMap<>());
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "tenant", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "tenant", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "tenant", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    private static final String[] TENANT_COLUMNS = new String[]{
            "account","audit_status","city","cmp_orders","code","contact_email","contact_name","contact_phone",
            "contract_begin","contract_finish","creator_id","discount","email","esop_orders","expired_date",
            "gmt_create","gmt_modify","group_code","id","is_deleted","last_login_date","last_pwd_modify_date",
            "login_status","manager_id","max_snapshot_count","mender_id","name","owner_id","parent_id","password",
            "payment_code","portrait","props","remark","session_id","status","town"
    };

    private static final String[] ACCOUNT_SECURITY_COLUMNS = new String[]{
            "target_id","salt","api_key","sec_key","tenant_id", "category"
    };
}
