package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 表结构无变化的统一迁移任务
 *
 * @author luyanchao
 * @date 2021-06-09
 */
@Component
public class UnchangedTask implements UpgradeTask {

    private static final Logger logger = LoggerFactory.getLogger(UnchangedTask.class);

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public UnchangedTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 获取5.0到5.5的用户id对应关系
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (String table : TABLES) {
            logger.debug("Start to migrate table: {}", table);
            // 先把数据从5.0迁移到5.5
            UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), table);
            // 获取5.5中的列列表
            List<String> columnNames = UpgradeUtils.getColumns(this.context.getBean(DataSource.class), table);
            for (Map.Entry<Long, Long> id : idMap.entrySet()) {
                for (String columeName : COLUMNS) {
                    if (columnNames.contains(columeName)) {
                        UpgradeUtils.updateColumn(this.context.getBean(DataSource.class),
                                table, columeName, id.getKey(), id.getValue());
                    }
                }
            }
            this.percent = Math.floorDiv(index * 100, TABLES.length);
            index++;
        }
        this.percent = 100;
    }

    @Override
    public String getName() {
        return "表结构无变化的数据迁移任务";
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    private static final String[] TABLES = new String[]{
            "account_record", "admission", "agile_cabinet", "agile_cabinet_group",
            "agile_device", "agile_device_group", "agile_device_link", "agile_device_port", "agile_dhcp_group",
            "agile_end_port", "agile_epg", "agile_epg_item", "agile_external_connect", "agile_external_gateway",
            "agile_external_network", "agile_fabric_pool", "agile_host", "agile_ilo_cabinet_pool", "agile_ilo_ip",
            "agile_ilo_ip_pool", "agile_ilo_message", "agile_ip", "agile_logic_port", "agile_logic_port_device_port",
            "agile_logic_port_ip", "agile_logic_router", "agile_logic_switch", "agile_logic_switch_business_bind",
            "agile_resource_pool_link", "agile_router_fabric_pool", "agile_sfc_contract", "agile_sfc_contract_rule",
            "agile_sfc_order", "agile_sfc_scapp", "agile_sfc_scapp_path_hop", "agile_sfc_template", "agile_sfc_type",
            "agile_static_router", "agile_subnet", "agile_tenant", "agile_tenant_dhcp_group",
            "agile_tenant_external_gateway", "agile_tenant_fabric_pool", "agile_tenant_vmm", "agile_vas",
            "agile_vas_link", "agile_vas_port", "agile_vpc", "agile_vpc_business_bind", "agile_vpc_connection",
            "agile_vpc_fabric_pool", "alarm_level", "alarm_threshold", "ass", "ass_node", "attachment", "auto_scaling",
            "auto_scaling_alarmtask", "auto_scaling_config", "auto_scaling_timedtask", "azure_resource_group",
            "batch_account", "bi_dns_resolvers", "bi_monitor", "bi_node", "bi_partition", "bi_pool", "bi_pool_member",
            "bi_profile", "bi_rule", "bi_snat_pool", "bi_tunnel", "bi_virtual_server", "bi_vlans", "bucket",
            "bucket_object", "business", "ccs", "ccs_node", "ceph_osd", /*"ci_category",*/ "ci_columns_config", "ci_file",
            "ci_history", /*"ci_icon",*/ "ci_instance", "ci_instance_grid", "ci_instance_location", "ci_instance_relation",
            "ci_instance_value", /*"ci_module",*/ /*"ci_module_relation",*/ /*"ci_property",*/ /*"ci_property_group",*/
            /*"ci_property_item",*/ /*"ci_regular_expression",*/ /*"ci_relation",*/ /*"ci_relation_category",*/ /*"ci_syn_config",*/
            "ci_syn_mapping", "cisco_bridge_domain", "cisco_contract", "cisco_encap_blk", "cisco_epg", "cisco_profile",
            "cisco_subject", "cisco_subnet", "cisco_tenant", "cisco_vlan_pool", "cisco_vrf", "clientdetails",
            "cloud_stack", "cloud_stack_event", "cluster", "cluster_node", "cluster_usage", "cms_log", "cms_node",
            "cms_topic", "command", "cos", "coupon", "data_store_file", "department", "disk_partition", "environment",
            "favorite", "file_client", "file_client_group", "file_client_group_relation", "file_gateway_group",
            "file_system", "file_user", "file_user_group", "file_user_group_relation", "firewall", "firewall_policy",
            "firewall_rule", "flash_copy", "flavor_extra_specs", "harbor", "harbor_image", "harbor_project",
            "harbor_tag", "hmc_hea_logical_port", "hmc_hea_physical_port", "hmc_host_ethernet_adapter",
            "hmc_io_adapter", "hmc_io_bus", "hmc_ipl_config", "hmc_partition", "hmc_partition_io",
            "hmc_partition_memory", "hmc_partition_network_adapter", "hmc_partition_processor",
            "hmc_partition_scsi_adapter", "hmc_partition_vfc_adapter", "hmc_physical_disk",
            "hmc_physical_fibre_channel", "hmc_profile", "hmc_profile_fibre_adapter", "hmc_profile_ioslot",
            "hmc_profile_lhea", "hmc_profile_network_adapter", "hmc_profile_scsi_adapter", "hmc_server",
            "hmc_system_capability", "hmc_system_memory_config", "hmc_system_processor_config",
            "hmc_system_shared_processor_pool", "hmc_virtual_disk", "hmc_virtual_fibre_channel_mapping",
            "hmc_virtual_network_bridge", "hmc_virtual_switch", "hmc_volume_group", "host_sea_adapter", /*"icon",*/
            "identification", "image_volume", "install_result", "install_task", "interactive_record",
            "ip_pool_port_group", "k8s_cluster", "k8s_node", "kube_application", "kube_container", "kube_namespace",
            "kube_node_label", "kube_persistent_volume", "kube_persistent_volume_claim", "kube_secret", "kube_service",
            "kvs_db", "kvs_log", "kvs_node", "kvs_param_log", "kvs_user", "lbs", "lbs_node", "log_archive_record",
            /*"login_record",*/ "manage_one_account", "manage_one_project", "manage_one_vdc", "mdisk",
            "mdisk_volume", "message", "message_record", "monitor_allocation_ratio", "monitor_analysis_rule",
            "monitor_analysis_snapshot", "monitor_auth", "monitor_host_ipmi", "monitor_ip_relation",
            /*"monitor_oracle_metric_config",*/ "monitor_resource", "name_rule", "network_host", "network_sea_adapter",
            "nsx_compute_manager", "nsx_edge_cluster", "nsx_firewall_rule", "nsx_firewall_section",
            "nsx_lb_server_relation", "nsx_lb_service", "nsx_logical_router", "nsx_logical_switch", "nsx_nat_rule",
            "nsx_ns_group", "nsx_router_port", "nsx_switch_port", /*"nsx_tier_zero",*/ "nsx_transport_zone",
            "nsx_vcenter_relation", "nsx_virtual_server", "oauth_access_token", "oauth_approvals",
            "oauth_client_details", "oauth_client_token", "oauth_code", "oauth_refresh_token", "object_relation",
            "object_router", "object_storage_container", "object_storage_item", "object_user", /*"order_sn",*/
            "os_tenant", "payment", "pbm_profile", "pool_group_label", "port_fixedip", "port_group",
            /*"portal_element_config",*/ /*"portal_element_pool",*/ /*"portal_panel",*/ "powervc_hmc", "powervc_hmc_server",
            "project", "qos_policy", "qos_policy_rules", "quota_history", "rds_log", "rds_node", "rds_param_log",
            /*"region",*/ "relation", "res_pool_server_rela", "resource_alarm", "resource_pool", /*"resource_quota", *//*"role",*/
            "role_command", "ros", "router_subnet", "rp_content_template", "rp_history_result",
            "rp_paragraph", "rp_paragraph_template_rela", "rp_publish_info", "rp_report_config_template",
            "rp_report_template_history", "san_fabric", "san_fabric_zone_member", /*"scheduler_policy",*/ /*"script",*/
            /*"script_content",*/ /*"script_group",*/ /*"script_version",*/ "scsi_lun", "server_assets", "server_config",
            "server_keypair", "server_pci", "server_raid_ctrl", "server_sgroup", "server_template", "server_volume",
            /*"service_category",*/ "service_role", /*"service_sla", */"service_tenant", "shopping_cart", /*"sku_spec",*/
            "sla_instance", "snapshot_group", "snapshot_task", "software", "software_group", "stack_template_version",
            "storage", "storage_access_path", "storage_access_path_gateway", "storage_fc_port", "storage_gateway",
            "storage_group", "storage_group_member", "storage_host_port", "storage_host_port_fcport",
            "storage_host_port_gateway", "storage_host_volume", "storage_iscsi_port", "storage_iscsi_port_gateway",
            "storage_node", "storage_raid", "storage_synced_set", "storage_view", "storage_view_group", "store_host",
            "target", "task_instance", "task_node", /*"task_node_module",*/ /*"task_node_module_group",*/ "task_node_relation",
            "task_node_template", /*"task_scene",*/ /*"task_template",*/ "template_version", "tenant_account", "tenant_network",
            "tenant_pool", "tenant_provider", "validate_result", "vdc", "vendor_sync_record",
            "volume_mirror", "volume_pool", "volume_template", "vpn", "vpn_conn", "vpn_policy", "wf_sequence", "work_order",
            "yum_repo", "zone"
    };

    private static final String[] COLUMNS = new String[]{"creator_id", "mender_id", "owner_id", "user_id"};
}
