package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * VPC数据迁移任务
 *
 * 5.0和5.5的数据完全一样，直接truncate5.5的数据，然后迁移5.0的数据就可以了
 *
 * @author luyanchao
 * @date 2021-06-17
 */
@Component
public class VpcTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id
     */
    @Autowired
    public VpcTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "VPC数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate jdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        // 截断5.5数据
        jdbcTemplate.update("delete from vpc", new HashMap<>());
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "vpc", COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "role", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "role", "mender_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        // 5.5相比于5.0增加了如下字段：
        // ipv6_cidr -- 王卫
        // router_uuid -- 袁明航
        // ddos_plan_uuid -- 李振平
        // ddos_plan_enabled -- 李振平
        // ddos_plan_id -- 李振平
        // resource_group_id -- 李振平
        // expired_time -- 袁明航
        // pool_group_id -- 袁明航
        // business_id -- 袁明航
        // 已经和对应人员确认留null
    }

    private static final String[] COLUMNS = new String[] {
            "id","uuid","name","status","router_id","vswitch_ids","region_id","cidr","vpc_id","remark","user_cidrs",
            "is_deleted","vendor_id","default_vpc","gmt_create","gmt_modify","creator_id","owner_id","mender_id",
            "props","tenant_id","project_id","azure_resource_group_name","network_id","catalog","resource_group_uuid"
    };
}
