package com.bocloud.sms.repository;

import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.sms.entity.Manager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/2/3 周一 11:38
 */

@Repository("managerRepository")
public class ManagerRepository extends JdbcGenericDao<Manager, Long> {

    @Autowired
    public ManagerRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 分页查询管理员
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @param roleId
     * @return
     * @throws Exception
     */
    public List<Manager> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long roleId)
            throws Exception {
        String sql;
        if (null != roleId) {
            sql = "select a.*,d.name depart_name from manager a left join manager_role b on a.id = b.manager_id left join department d on d.id = a.depart_id where a.id != 1 and a.is_deleted = 0 and b.role_id = "
                    + roleId;
        } else {
            sql = "select a.*,d.name depart_name from manager a left join department d on d.id = a.depart_id where a.id != 1 and a.is_deleted = 0 ";

        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(Manager.class, sql, paramMap);
    }

    /**
     * 简易查询
     *
     * @param params
     * @param sorter
     * @param roleId
     * @return
     * @throws Exception
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long roleId) throws Exception {
        String sql;
        if (null != roleId) {
            sql = "select a.*,d.name depart_name from manager a left join manager_role b on a.id = b.manager_id left join department d on d.id = a.depart_id where a.id != 1 and a.is_deleted = 0 and b.role_id = "
                    + roleId;
        } else {
            sql = "select a.*,d.name depart_name from manager a left join department d on d.id = a.depart_id where a.id != 1 and a.is_deleted = 0";
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<Manager> managers = this.list(Manager.class, sql, paramMap);
        return managers.stream().map(manager -> new SimpleBean(manager.getId(), manager.getName())).collect(Collectors.toList());
    }

    /**
     * 查询所有管理员
     *
     * @return
     * @throws Exception
     */
    public List<Manager> list() throws Exception {
        String sql = "select * from manager where is_deleted = 0";
        return this.list(Manager.class, sql);
    }

    /**
     * 查询有过期时间的管理员
     *
     * @return
     * @throws Exception
     */
    public List<Manager> listManagerHasExpiredDate() throws Exception {
        String sql = "select * from manager where is_deleted = 0 and expired_date is not null";
        return this.list(Manager.class, sql);
    }

    /**
     * 根据组织机构ID查询
     *
     * @param departId
     * @return
     * @throws Exception
     */
    public List<Manager> listByDepartment(Long departId) throws Exception {
        String sql = "select a.* from manager a where a.is_deleted = 0 and a.depart_id = :departId";
        Map<String, Object> paramMap = MapTools.simpleMap("departId", departId);
        return this.list(Manager.class, sql, paramMap);
    }

    /**
     * 删除管理员
     *
     * @param id
     * @param menderId
     * @return
     * @throws Exception
     */
    public boolean delete(Long id, Long menderId) throws Exception {
        String sql = "update manager set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 锁定管理员
     *
     * @param id
     * @param menderId
     * @return
     * @throws Exception
     */
    public boolean lock(Long id, Long menderId) throws Exception {
        return this.handleStatus(id, menderId, BaseStatus.ABNORMAL);
    }

    /**
     * 激活管理员
     *
     * @param id
     * @param managerId
     * @return
     * @throws Exception
     */
    public boolean active(Long id, Long managerId) throws Exception {
        return this.handleStatus(id, managerId, BaseStatus.NORMAL);
    }

    private boolean handleStatus(Long id, Long actor, BaseStatus status) throws Exception {
        String sql = "update manager set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", actor);
        params.put("status", status.name());
        return this.execute(sql, params) > 0;
    }

    /**
     * 查询管理员数量
     *
     * @param params
     * @param roleId
     * @return
     * @throws Exception
     */
    public int count(List<Param> params, Long roleId) throws Exception {
        String sql;
        if (null != roleId) {
            sql = "select count(a.id) from manager a left join manager_role b on a.id = b.manager_id where a.id != 1 and a.is_deleted = 0 and b.role_id = "
                    + roleId;
        } else {
            sql = "select count(a.id) from manager a where a.id != 1 and a.is_deleted = 0";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 获取管理员信息
     *
     * @param id
     * @return
     * @throws Exception
     */
    public Manager query(Long id) throws Exception {
        String sql = "select * from manager where id = :id and is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<Manager> list = this.list(Manager.class, sql, paramMap);
        Assert.isTrue(list.size() <= 1, "more than one manager in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据账号查询
     *
     * @param account
     * @return
     * @throws Exception
     */
    public Manager getByAccount(String account) throws Exception {
        String sql = "select * from manager where account = :account and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ACCOUNT, account);
        List<Manager> list = this.list(Manager.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one manager in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询具有指定权限菜单的管理员集合
     *
     * @param router
     * @return
     * @throws Exception
     */
    public List<Manager> listByPermission(String router) throws Exception {
        String sql = "select distinct m.* from manager m left join manager_role mr on m.id = mr.manager_id left join role r on r.id = mr.role_id and r.is_deleted = 0 "
                + "left join role_permission rp on r.id = rp.role_id left join permission p on p.id = rp.permission_id where m.is_deleted = 0  and p.router =:router";
        Map<String, Object> paramMap = MapTools.simpleMap("router", router);
        return this.list(Manager.class, sql, paramMap);
    }

    public List<Manager> listByOrigin(Long departId, String origin, boolean hasDept) throws Exception {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.origin = :origin";
        Map<String, Object> paramMap = MapTools.simpleMap("origin", origin);
        if (hasDept) {
            if (null != departId) {
                sql += " and a.depart_id = :departId";
                paramMap.put("departId", departId);
            }
        } else {
            sql += " and a.depart_id is null";
        }
        return this.list(Manager.class, sql, paramMap);
    }

    /**
     * 用户登录信息总数
     *
     * @param params
     * @return
     * @throws Exception
     */
    public int loginTimes(List<Param> params) {
        String sql = "select count(id) from login_record where category = :category";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> paramsMap = MapTools.simpleMap("category", RequestContext.Catalog.Manager.name());
        return Integer.parseInt(super.countQuery(sql,paramsMap).toString());
    }

    /**
     * 根据管理员ID解绑管理员角色
     *
     * @param managerId
     * @return
     * @throws Exception
     */
    public boolean unBindRole(Long managerId) throws Exception {
        String sql = "delete from manager_role where manager_id = :managerId";
        Map<String, Object> params = MapTools.simpleMap("managerId", managerId);
        return super.execute(sql,params) > 0;
    }
}
