package com.bocloud.sms.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.sms.entity.Project;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/6/2 周二 15:57
 */
@Repository("projectRepository")
public class ProjectRepository extends JdbcGenericDao<Project, Long> {

    private static final Logger logger = LoggerFactory.getLogger(ProjectRepository.class);

    public ProjectRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Project> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (params.size() == 0) {
            sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.is_deleted = 0 and a.tenant_id is not null";
            sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
            return this.list(Project.class, sql);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(Project.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select a.id,a.name from project a where is_deleted = 0 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        return this.list(Project.class, sql, paramMap).stream().map(project -> new SimpleBean(project.getId(), project.getName())).collect(Collectors.toList());
    }

    public Project query(Long id) throws BoCloudDBException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.id = :id and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Project> list = this.list(Project.class, sql, params);
        Assert.isTrue(list.size() <= 1, "project more than one in table");
        return list.isEmpty() ? null : list.get(0);
    }

    public List<Project> listAll() throws BoCloudDBException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.is_deleted = 0";
        return  this.list(Project.class, sql);
    }

    public boolean remove(Long id, Long userId) throws BoCloudDBException {
        String sql = "update project set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean batchRemove(List<Long> ids, Long userId) throws BoCloudDBException {
        String sql = "update project set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id in (:ids) ";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "select count(1) from project where is_deleted = 0";
        Map<String, Object> param = QueryBuilder.getParam(params);
        if(params.size() == 0){
            sql = "select count(1) from project where is_deleted = 0 and tenant_id is not null";
            return this.countQuery(sql,null).intValue();
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, param).intValue();
    }

    public List<Project> listUserProject(Long userId) throws BoCloudDBException {
        String sql = "select distinct a.* from project a left join user_project b on a.id = b.project_id where a.is_deleted = 0 and b.user_id = :userId";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        return this.list(Project.class, sql, paramMap);
    }

    public List<Project> listUserManagerProjects(Long userId) throws BoCloudDBException {
        String sql = "select distinct a.* from project a left join user_project b on a.id = b.project_id where a.is_deleted = 0 and b.user_id = :userId and b.project_admin = 1";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        return this.list(Project.class, sql, paramMap);
    }

    public List<Project> listUserVisibleProjects(Long userId) throws BoCloudDBException {
        String sql = "select distinct a.* from project a left join user_project b on a.id = b.project_id where a.is_deleted = 0 and b.user_id = :userId and a.visibility != 'invisible'  ";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        return this.list(Project.class, sql, paramMap);
    }

    public List<Project> listTenantProject(Long tenantId) throws BoCloudDBException {
        String sql = "select a.* from project a where a.is_deleted = 0 and a.tenant_id = :tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(Project.class, sql, paramMap);
    }

    public boolean exist(String name) throws BoCloudDBException {
        String sql = "select a.* from project a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<Project> result = this.list(Project.class, sql, paramMap);
        return !result.isEmpty();
    }

    public List<Project> businessProjects(Long businessId) throws BoCloudDBException {
        String sql = "select * from project where business_id =:businessId and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("businessId", businessId);
        return this.list(Project.class, sql, params);

    }

    /**
     * 查询项目列表
     *
     * @param tenantId
     * @param visibility
     * @return
     * @throws BoCloudDBException
     */
    public List<Project> list(Long tenantId, String visibility) throws BoCloudDBException {
        String sql = "select * from project where is_deleted = 0 and tenant_id =:tenantId and visibility = :visibility";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("visibility", visibility);
        return this.list(Project.class, sql, params);
    }

    /**
     * 导出项目列表
     *
     * @param params
     * @param projectIds
     * @return
     * @throws BoCloudDBException
     */
    public List<Project> export(List<Param> params, List<Long> projectIds) throws BoCloudDBException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.is_deleted = 0";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.id in (:projectIds)";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, null), "a");
        return this.list(Project.class, sql, paramMap);
    }

    /**
     * 根据业务ID查询
     *
     * @param businessId
     * @return
     * @throws BoCloudDBException
     */
    public List<Project> listByBusiness(Long businessId) throws BoCloudDBException {
        String sql = "select * from project where is_deleted = 0 and business_id =:businessId";
        Map<String, Object> params = MapTools.simpleMap("businessId", businessId);
        return this.list(Project.class, sql, params);
    }

    /**
     * 根据业务id查询
     *
     * @param businessId
     * @param tenantId
     * @return
     * @throws BoCloudDBException
     */
    public List<Project> listByBusiness(Long businessId, Long tenantId) throws BoCloudDBException {
        String sql = "select * from project where is_deleted = 0 and business_id =:businessId and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("businessId", businessId);
        params.put("tenantId", tenantId);
        return this.list(Project.class, sql, params);
    }


    /**
     * 根据项目id查询
     *
     * @param projectIds
     * @return
     * @throws BoCloudDBException
     */
    public List<Project> listByIds(List<Long> projectIds) throws BoCloudDBException {
        String sql = "select * from project where is_deleted = 0 and id in (:projectIds)";
        Map<String, Object> params = MapTools.simpleMap("projectIds", projectIds);
        return this.list(Project.class, sql, params);
    }

    /**
     * 根据编码查询
     *
     * @param code
     * @return
     * @throws BoCloudDBException
     */
    public Project getByCode(String code) throws BoCloudDBException {
        String sql = "select a.* from project a where a.code = :code  and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<Project> list = this.list(Project.class, sql, params);
        Assert.isTrue(list.size() <= 1, "project more than one in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 获取没有配置业务的项目
     *
     * @throws  BoCloudDBException
     * @return
     */
    public List<Project> getNoBusinessProjects() throws BoCloudDBException {
        String sql = "select  * from project where is_deleted = 0 and business_id is null ";
        return this.list(Project.class, sql);
    }
}

