package com.bocloud.sms.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.sms.entity.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/2/3 周一 11:41
 */
@Repository("roleRepository")
public class RoleRepository extends JdbcGenericDao<Role, Long> {

    @Autowired
    public RoleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);

    }

    /**
     * 根据ID删除角色
     *
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean delete(Long id, Long menderId) throws BoCloudDBException {
        String sql = "update role set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据用户ID查询角色
     *
     * @param userId
     * @return
     * @throws BoCloudDBException
     */
    public List<Role> listByUser(Long userId) throws BoCloudDBException {
        String sql = "select r.* from role r,user_role u where u.user_id= :userId and r.id = u.role_id and r.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        return this.list(Role.class, sql, params);
    }

    public void batchRemoveApiPermission(Long roleId) {
        String sql = "delete from api_permission where role_id = :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        super.execute(sql,params);
    }


    /**
     * 根据管理员ID查询角色
     *
     * @param managerId
     * @return
     * @throws BoCloudDBException
     */
    public List<Role> listByManager(Long managerId) throws BoCloudDBException {
        String sql = "select r.* from role r,manager_role mr where mr.manager_id= :managerId and r.id = mr.role_id and r.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("managerId", managerId);
        return this.list(Role.class, sql, params);
    }

    /**
     * 查询所有管理员角色
     *
     * @return
     * @throws BoCloudDBException
     */
    public List<Role> listManagerRole() throws BoCloudDBException {
        String sql = "select r.* from role r where tenant_id =0 and r.is_deleted = 0";
        return this.list(Role.class, sql);
    }

    /**
     * 分页查询
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     * @throws BoCloudDBException
     */
    public List<Role> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from role where is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.list(Role.class, sql, param);
    }

    public List<Role> list() throws BoCloudDBException {
        String sql = "select * from role where is_deleted = 0";
        return this.list(Role.class, sql);
    }

    /**
     * 获取角色数量
     *
     * @param params
     * @return
     * @throws BoCloudDBException
     */
    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "select count(r.id) from role r where r.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "r");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    /**
     * 查询指定租户下所有角色
     *
     * @param tenantId
     * @return
     * @throws BoCloudDBException
     */
    public List<Role> listRole(Long tenantId) throws BoCloudDBException {
        String sql = "select * from role where is_deleted = 0 and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(Role.class, sql, params);
    }

    /**
     * 根据ID查询
     *
     * @param id
     * @return
     * @throws BoCloudDBException
     */
    public Role query(Long id) throws BoCloudDBException {
        String sql = "select * from role where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<Role> list = this.list(Role.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one role in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据名称查询角色
     *
     * @param name
     * @param tenantId
     * @return
     * @throws BoCloudDBException
     */
    public Role exists(String name, Long tenantId) throws BoCloudDBException {
        String sql = "select * from role where is_deleted = 0 and name = :name and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("tenantId", tenantId);
        List<Role> list = this.list(Role.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one role in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据租户ID查询角色
     *
     * @param tenantId
     * @return
     * @throws BoCloudDBException
     */
    public List<Role> listByTenant(Long tenantId) throws BoCloudDBException {
        String sql = "select * from role a where a.tenant_id= :tenantId and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(Role.class, sql, params);
    }

    /**
     * 根据租户ID删除角色
     *
     * @param tenantId
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean deleteByTenant(Long tenantId, Long menderId) throws BoCloudDBException {
        String sql = "update role set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 简易查询管理员角色
     *
     * @param params
     * @param sorter
     * @return
     * @throws BoCloudDBException
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from role where is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        List<Role> roles = this.list(Role.class, sql, param);
        return roles.stream().map(role -> new SimpleBean(role.getId(), role.getName())).collect(Collectors.toList());
    }

    /**
     * 根据角色ID删除管理员角色
     *
     * @param roleId
     * @return
     * @throws BoCloudDBException
     */
    public boolean unBindManager(Long roleId) throws BoCloudDBException {
        String sql = "delete from manager_role  where role_id= :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        return super.execute(sql,params) > 0;
    }

    /**
     * 根据角色ID解绑用户角色
     *
     * @param roleId
     * @return
     * @throws BoCloudDBException
     */
    public boolean unBindUser(Long roleId) throws BoCloudDBException {
        String sql = "delete from user_role  where role_id= :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        return super.execute(sql,params) > 0;
    }

    /**
     * 解绑服务角色关联
     *
     * @param roleId
     * @return
     * @throws BoCloudDBException
     */
    public boolean unBindService(Long roleId) throws BoCloudDBException {
        String sql = "delete from service_role  where role_id= :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        return super.execute(sql,params) > 0;
    }

    /**
     * 解绑租户下角色服务关联
     *
     * @param tenantId  租户id
     * @param serviceId 服务id
     * @return
     * @throws BoCloudDBException
     */
    public boolean unBindService(Long tenantId, Long serviceId) throws BoCloudDBException {
        String sql = "delete from service_role where service_id = :serviceId and role_id in ( select role.id from  role where tenant_id=:tenantId and is_deleted = 0) ";
        Map<String, Object> params = MapTools.simpleMap("serviceId", serviceId);
        params.put("tenantId", tenantId);
        return super.execute(sql,params) > 0;
    }

    public List<Role> listTenantApiRole(Long tenantId) throws BoCloudDBException {
        String sql = "select DISTINCT r.* from role r LEFT JOIN api_permission  ap on  ap.role_id = r.id where r.props = 'default' and r.is_deleted = 0 and r.tenant_id= :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(Role.class, sql, params);
    }

    public List<Role> listManagerApiRole(Long managerId) throws  BoCloudDBException{
        String sql = "SELECT DISTINCT  a.* FROM role a LEFT JOIN manager_role mr on a.id = mr.role_id LEFT JOIN api_permission ap on  ap.role_id = a.id WHERE a.is_deleted  = 0 and mr.manager_id = :managerId";
        Map<String, Object> params = MapTools.simpleMap("managerId",managerId);
        return this.list(Role.class, sql, params);
    }

    public List<Role> getTenantApiRole() {
        String sql = "SELECT a.* FROM role a  WHERE a.is_deleted  = 0 and a.props ='tenantAPi' ";
        return this.list(Role.class, sql);
    }
    /**
     * 根据租户ID查询租户默认角色
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    public Role getDefaultRole(Long tenantId) {
        String sql = "select * from role where is_deleted = 0 and tenant_id = :tenantId and props ='default'";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        List<Role> list = super.list(Role.class,sql,params);
        Assert.isTrue(list.size() <= 1, "more than one role in the table");
        return list.isEmpty() ? null : list.get(0);
    }
}
